% Simulationsumgebung für den Wabenbau sozialer Insekten

global DX;
global DY;

DX = 31; % Breite des Gitters
DY = 31; % Höhe des Gitters
wespenanzahl = 1; % Zahl der aktiven Agenten
MaxSteps = 100;

global Wabengitter;
Wabengitter = hextop(DX,DY); % Erzeugen des hexagonalen Gitters
% Struktur der Matrix:
% Zeile 1: X-Koordinate des Knotenpunkts
% Zeile 2: Y-Koordinate des Knotenpunkts
% Zeile 3: Markierung, ob Knotenpunkt Wabenmittelpunkt ist (1/0)
% Zeile 4: Belegung des Feldes (0 = leer, 1 = Baustein Typ 1, ...)

global Knotenzahl;
Knotenzahl = size(Wabengitter,2); 

Position = DX + 2; % Initialisierung - erster Wabenmittelpunkt
global zeit;
global Grenzen;
global MittelPunkte;
global MittelWabe;
global raumplot;
global bebautezellen;

        
% Algorithmus zur Markierung der Wabenmittelpunkte
% - legt eine dritte Zeile in der Matrix Wabengitter an und markiert jeden
% Knotenpunkt, der einem Wabenmittelpunkt entspricht, mit einer 1

for Position=DX+2:3:2*DX-1 % die zweite Reihe vom zweiten bis zum vorletzten Element
                           % mit horizontalem Abstand 3
    Position2 = Position;
    while Position2 < (Knotenzahl - DX) % oberste Zeile ausschließen
          Wabengitter(3,Position2) = 1; % Setze Markierung für Mittelpunkt
          Position2 = Position2 + 2 * DX;
    end
    Position2 = Position + DX + 2;
    if (mod(Position2,DX) > 1) && (mod(Position2,DX) < DX) 
       while Position2 < Knotenzahl - DX
             Wabengitter(3,Position2) = 1;
             Position2 = Position2 + 2*DX;
       end    
    end  
end
            
MittelPunkte = find(Wabengitter(3,:)); 
  % erzeugt einen Vektor mit Indizes der Wabenmittelpunkte
Grenzen = max(Wabengitter,[],2); 
  % ereugt einen Vektor, der die Endkoordinaten des Gitters enthält
MittelWabe = MittelPunkte(ceil(length(MittelPunkte) / 2));
  % Index der annähernd mittleren Wabe (Ausgangspunkt für die Simulation)

scrsz = get(0,'ScreenSize');
% erzeuge figure für die graphische Anzeige
visual = figure('Name','Simulation', ...
                'Position',[1 scrsz(4) scrsz(3) scrsz(4)]);
raumplot = gca;
hold on; % verhindern, dass gezeichnetes wieder gelöscht wird



view(3); % Betrachtungswinkel einstellen
axis(raumplot,'manual');
set(raumplot, 'DataAspectRatio', [1 1 1] ); % , ...
         % 'PlotBoxAspectRatio', [1 1 1] );
axis(raumplot,[-2 (Grenzen(1)+2) -2 (Grenzen(2)+2)]);
camlight('right');
camproj('perspective');
lighting gouraud;

% leeres Gitter anzeigen
plotgitter(Wabengitter, raumplot);

Wabengitter(4,MittelWabe) = 1; % Baustein vom Typ 1 an Mittelpunkt gesetzt
                    % Wird als Anfangspunkt benötigt
plotwabe(MittelWabe, raumplot, 8); % die Anfangswabe wird markiert


for nummer=1:wespenanzahl
    wespe(nummer) = bauagent(Wabengitter(1:2, MittelWabe)', nummer); 
      % erzeugt ein Objekt der Klasse Bauagent auf den Koordinaten
      % der mittleren Wabe
end
  % erzeugt die durch wespenanzahl festgelegte Menge von Objekten der
  % Klasse 'agent'
  
  



% Simulationsstart
bebautezellen = find(Wabengitter(4,:)); % Update der Liste der Indizes aller bebauten Zellen

grafik = gcf;
% moviefile = avifile('simulation_04.avi');
% moviefile.Quality = 100;
% moviefile.fps = 5;

for zeit=uint32(1):MaxSteps % aktuelle Simulationszeit als Integerwert - wird in jedem Zeitschritt um 1 erhöht 
    for wespennummer=1:wespenanzahl
       wespe(wespennummer).movebuild();
    end
    % plotwaben(Wabengitter);
    % drawnow;
    
    
    % film = getframe(grafik); % videoaufzeichnung
    % moviefile = addframe(moviefile,film);
    
    % displayString = ['Zeit: ' num2str(Zeit) ];
    % text(5,5,displayString);
end

% close(grafik);
% moviefile = close(moviefile);

[fitnesswert, idealwert, aussenlaenge, wabenanzahl, differenz] = fitness(Wabengitter);
outp = sprintf('outputargumente %d \n fitness %f \n idealwert %f \n aussenlaenge %d \n wabenzahl %d \n differenz %f', ...
        nargout, fitnesswert, idealwert, aussenlaenge, wabenanzahl, differenz)
    




        